@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -717,6 +717,7 @@ void Document::RemoveBond (Bond* pBond)
 	m_pView->Update (pAtom1);
 	pMol->Lock (false);
 	if (pBond->IsCyclic ()) {
+		pBond->RemoveAllCycles ();
 		pMol->Remove (pBond);
 		pMol->UpdateCycles ();
 		Update ();
@@ -943,6 +944,7 @@ void Document::FinishOperation ()
 	}
 	Update ();
 	m_NewObjects.clear ();
+	EmptyTranslationTable ();
 }
 
 void Document::AbortOperation()
@@ -1025,7 +1027,8 @@ void Document::LoadObjects (xmlNodePtr node)
 		child = child->next;
 	}
 	//Now, add bonds
-	m_bIsLoading = true;
+	if (!m_bUndoRedo) /* this looks unsafe but needed at least when undoing partial selection ops */
+		m_bIsLoading = true;
 	child = GetNodeByName (node, "bond");
 	while (child) {
 		Bond* pBond = new Bond ();
@@ -1075,12 +1078,14 @@ void Document::PasteData (xmlNodePtr node)
 		child = (strcmp ((const char*) node->name, "object"))? node: node->children;
 		str = (const char*) child->name;
 		pObject = app->CreateObject (str, this);
-		AddObject (pObject);
-		if (!pObject->Load (child))
-			Remove (pObject);
-		else {
-			m_pView->Update (pObject);//FIXME: should not be necessary, but solve problem with cyclic double bonds
-			pData->SetSelected (pObject);
+		if (pObject) {
+			AddObject (pObject);
+			if (!pObject->Load (child))
+				Remove (pObject);
+			else {
+				m_pView->Update (pObject);//FIXME: should not be necessary, but solve problem with cyclic double bonds
+				pData->SetSelected (pObject);
+			}
 		}
 		node = node->next;
 	}
@@ -1410,4 +1415,9 @@ gcu::Residue *Document::CreateResidue (char const *name, char const *symbol, gcu
 	return res;
 }
 
+gcu::Window *Document::GetWindow ()
+{
+	return m_Window;
+}
+
 }	//	namespace gcp
